/*==============================================================================*/
/*  Copyright (C) 2009-2016, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   Data Writing Driver                                              */
/*  EventID:   Insert Kernelinfo Test Point                                     */
/*  AppVer :   DT10 `Ver10.51@                                                */
/*  FileID :   39C2                                                             */
/*  Version:   1.1                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>

#define DT10_READ_BUFF_SIZE	4096
#define CONVERT_FILE		"/home/hldc0055/ConvertData.DAT"	/* Customize */
#define DT10_DRV		"/proc/dt10proc"

#define DT10_IOCTL_MAGIC	'd'
#define DT10_IOCTL_TRACE_START			_IO(DT10_IOCTL_MAGIC, 0)
#define DT10_IOCTL_TRACE_STOP			_IO(DT10_IOCTL_MAGIC, 1)
#define DT10_IOCTL_BUFFER_CLEAR			_IO(DT10_IOCTL_MAGIC, 2)
#define DT10_IOCTL_TRACE_SETTING_BUFF_SIZE	_IOW(DT10_IOCTL_MAGIC, 3, int)
#define DT10_IOCTL_TRACE_SETTING_RING_BUFF	_IOW(DT10_IOCTL_MAGIC, 4, int)
#define DT10_IOCTL_MAXNR	6

enum dt10_mode {
	TRACE_STOP = 0,
	TRACE_START,
	SETTING_BUFF_CLEAR
};

int fh = -1;

int convert_data(void)
{
	FILE *fd = NULL;
	int len = 0, tmp = 0;
	char buff[DT10_READ_BUFF_SIZE];

	memset(buff, 0x0, DT10_READ_BUFF_SIZE); 

	fd = fopen(CONVERT_FILE, "wb");
	if (fd == NULL) {
		printf("[dt10]can't open convert file: %s\n", CONVERT_FILE);
		return -1;
	}
	do {
		tmp = read(fh, buff, DT10_READ_BUFF_SIZE);
		if (tmp > 0) {
			fwrite(buff, sizeof(char), tmp, fd);
			len += tmp;
		}
	} while (tmp > 0);
	printf ("\n >>>%d bytes saved.\n", len);
	fclose(fd);

	return len;
}

/* argv[0]: mode setting(1: trace start. 0: trace stop) */
/* argv[1]: buffer size setting(set MB) default is 0. */
/* argv[2]: ring buffer setting(1: use ring buffer. 0: don't use ring buffer) default is 0. */
int main(int argc, char* argv[])
{
	int i;
	unsigned int size, tmp = 0, mode;
	int use_ring = -1;

	fh = open(DT10_DRV,O_RDWR);
	if (fh <= 0) {
		printf("[dt10]can't open file\n");
		return -1;
	}

	switch (argc) {
	case 4:
		use_ring = atoi(argv[3]);
	case 3:
		size = atoi(argv[2]) * 1024 * 1024;	/* Convert Byte */
		tmp = size - (size % 18);
	case 2:
		mode = atoi(argv[1]);
		if (mode == 1) {
			if (argc == 4) {
				ioctl(fh, DT10_IOCTL_TRACE_SETTING_RING_BUFF, use_ring);
				printf("Ring Buffer: %s\n", atoi(argv[3])?"Use":"Not Use");
			}
			if (argc >= 3) {
				ioctl(fh, DT10_IOCTL_TRACE_SETTING_BUFF_SIZE, tmp);
				printf("vmalloc_size: %dByte.\n", tmp);
			}
			ioctl(fh, DT10_IOCTL_TRACE_START);
			printf("mode: Trace Start.\n");
		} else if (mode == 0) {
			ioctl(fh, DT10_IOCTL_TRACE_STOP);
			printf("mode: Trace Stop.\n");
		} else {
			ioctl(fh, DT10_IOCTL_BUFFER_CLEAR);
			printf("mode: Buffer Clear.\n");
		}
		break;
	case 1:
		convert_data();
		break;
	default:
		printf("Wrong Input Type.\n");
		break;
	}
	close(fh);

	return 1;
}
